# Log FilterProxy Module
#
# Saves a log of URLs
#
# Seth Golub <seth@aigeek.com>
# http://www.aigeek.com/geek/autojot/
# 23 Jul 2000
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

package FilterProxy::Log;

use strict;
no strict 'subs';

use IO::File qw( SEEK_END );
use Fcntl qw( :flock );

use vars qw($VERSION $CONFIG);  # The $CONFIG variable MUST be exported.
                                # and should be a reference to a hash.
                                # keys to this hash are configuration
                                # variables, and values are used by this 
                                # module.

push @FilterProxy::MODULES,     # add myself to FilterProxy's list of known 
   "Log";                       # modules
$VERSION = 0.0.2;
$CONFIG = {};                   # my $CONFIG.  This will be saved to and
                                # loaded from the config file by FilterProxy
                                # Module should define a basic CONFIG just in
                                # case this is the first time this module
                                # has ever been used.

$CONFIG->{logfile} = "$ENV{HOME}/.autojot/urls";

$CONFIG->{order} = 1;           # must be a value in the range -10..10
                                # which specifies where in the chain of
                                # filters it wants to be executed.
                                # < 0: called before request is sent
                                # > 0: called after content is received
                                # -10    Outgoing header manglers
                                # -9..-1 Outgoing content manglers.
                                # -9..-7 May decode outgoing content.
                                # -6..-4 May modify outgoing content.
                                # -3..-1 May encode outgoing content.
                                # 0      not used
                                #  1..3: May decode incoming content
                                #  4..6: May modify incoming content
                                #  7..9: May encode incoming content
                                # 10: Must be called after all filters. 
                                #   (final header manipulation)
                                # This may also be an anonymous list ref if
                                # the module needs to be called more than once
                                # for a given request. (i.e. header mungers)
                                # 1-9 can expect content, 10 should not 
                                # depend on it.

$CONFIG->{mime_types} = [       # is a list of mime-types that this module
  "text/html",                  # knows how to filter.  Can contain an empty
  "text/plain",                 # string to specify all mime-types.
];

$CONFIG->{ignore_urls} = [];


sub match_any
{
  return 0;
  my ( $text, $patterns ) = @_;
  foreach my $pattern ( @$patterns )
    {
      return 1 if $text =~ m@$pattern@i;
    }
  return 0;
}

sub filter {                    # The main filtering routine.  Does the dirty work.
  my $request    = shift;       # the HTTP::Request object 
  my $response   = shift;       # the HTTP::Response object.  
  my $siteconfig = shift;

  FilterProxy::logger(DEBUG, "FilterProxy::Log::filter called.");

  if ( !defined( $response ) || !ref( $response ) ) { return; }
  my $code = $response->code();

  return unless ( $code == 200 || $code == 201 || $code == 202 );
  return unless grep($response->content_type, $CONFIG->{mime_types});
  my $url = $response->base();
  return if match_any( $url, $CONFIG->{ignore_urls} );

  my $fh = new IO::File;

  # It might be faster to leave the log file open all the time.
  $fh->open( ">>$CONFIG->{logfile}" );  # Open log file
  flock( $fh, LOCK_EX );      # wait for exclusive lock
  $fh->seek( 0, SEEK_END );   # seek to end
  $fh->print( "$url\n" );     # append new url
  flock( $fh, LOCK_UN );      # unlock
  $fh->close();               # close log file
}

sub Config {
  my($req, $cgi, $siteconfig) = @_;
  if($cgi->param()) {
    if(defined $cgi->param('logfile')) {
      $CONFIG->{logfile} = $cgi->param('logfile');
    }
    if(defined $cgi->param('mime_types')) {
      $CONFIG->{mime_types} = [split(/\s*[\r\n]+/, $cgi->param('mime_types'))];
    }
    if(defined $cgi->param('ignore_urls')) {
      $CONFIG->{ignore_urls} = [split(/\s*[\r\n]+/, $cgi->param('ignore_urls'))];
    }
  }
  return "It worked.\n";
}

# NOTES:
#
# Is it advantageous to make FilterProxy::Module, and make everything an object?
